@@ -18,7 +18,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: GCConfig.py 7050 2005-02-21 14:13:54Z kilo $
+# $Id: GCConfig.py 7637 2005-06-22 09:50:24Z johannes $
 
 """
 Valid configuration options that apply to all GNUe tools
@@ -26,6 +26,8 @@ Valid configuration options that apply to all GNUe tools
 can appear in each individual tool section)
 """
 
+import locale
+
 from gnue.common.formatting import GTypecast
 
 ConfigOptions = (
@@ -36,7 +38,7 @@ ConfigOptions = (
     'Description': 'Encoding for XML headers and for fonts in forms. '
                  + 'Like iso8859-1, iso8859-13.',
     'Typecast'   : GTypecast.text,
-    'Default'    : 'iso8859-1' },
+    'Default'    : locale.getpreferredencoding ()},
         
   { 'Name'       : 'ImportPath',
     'Type'       : 'Setting',
