@@ -19,18 +19,18 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: GBaseApp.py 7437 2005-04-20 16:18:41Z reinhard $
+# $Id: GBaseApp.py 7898 2005-09-12 14:31:40Z jamest $
 """
 Class that provides a basis for GNUe applications.
 
 Typically, this class will not be called; rather, a tool will be a GClientApp
 or GServerApp.
 """
+__revision__ = "$Id"
 
 import sys
 import os
 import getopt
-import string
 import types
 import ConfigParser
 import gc
@@ -40,7 +40,7 @@ from gnue import paths
 
 from gnue.common.apps import errors, GConfig, GDebug
 from gnue.common.datasources import GConnections
-from CommandOption import CommandOption
+from gnue.common.apps.CommandOption import CommandOption
 
 
 # =============================================================================
@@ -48,6 +48,10 @@ from CommandOption import CommandOption
 # =============================================================================
 
 class StartupError (errors.UserError):
+  """
+  Error raised when a gnue application fails during initial startup prior to 
+  initializing to the point that better error handlers are available.
+  """
   pass
 
 
@@ -265,7 +269,7 @@ class GBaseApp:
     #       be more efficent in mod_python cases
     #       But not this close to a release :)
     if not sys.__dict__.has_key('argv'):
-      sys.argv=[]
+      sys.argv = []
 
     try:
       opt, self.ARGUMENTS = getopt.getopt (sys.argv[1:], shortoptions,
@@ -325,11 +329,11 @@ class GBaseApp:
                        self.OPTIONS ['debug-file'])
     except ValueError:
        raise StartupError, \
-           _('The debug_level option ("-d") expects numerical values.')
+          _('The debug_level option ("-d") expects numerical values.')
 
-    gDebug (2, "Python %s" % sys.version)
-    gDebug (2, "Run Options: %s" % opt)
-    gDebug (2, "Run Arguments: %s" % self.ARGUMENTS)
+    assert gDebug (2, "Python %s" % sys.version)
+    assert gDebug (2, "Run Options: %s" % opt)
+    assert gDebug (2, "Run Arguments: %s" % self.ARGUMENTS)
 
     # Read the config files
     if application:
@@ -347,14 +351,14 @@ class GBaseApp:
     except:
       extrapaths = ""
     if extrapaths:
-      for path in string.split(extrapaths,','):
-        p = string.strip(path)
+      for path in extrapaths.split(','):
+        p = path.strip()
         if not p in sys.path:
           sys.path.append(p)
 
     # Get the connection definitions
     if connections != None:
-      gDebug(7,"Reusing connections instance")
+      assert gDebug(7,"Reusing connections instance")
       self.connections = connections
     elif self.USE_CONNECTIONS:
 
@@ -373,7 +377,7 @@ class GBaseApp:
       else:
         self.connections_file = os.path.join (paths.config, "connections.conf")
 
-      gDebug(2, 'Connection Definition: "%s"' % self.connections_file)
+      assert gDebug(2, 'Connection Definition: "%s"' % self.connections_file)
 
       try:
         self.connections = GConnections.GConnections (self.connections_file,
@@ -383,7 +387,7 @@ class GBaseApp:
             u_("Unable to load the connections definition file.\n\n"
                "The connections file is in an invalid format.\n%s") % msg
 
-      except IOError, err:
+      except IOError:
         raise StartupError, \
             u_("Unable to load the connections definition file: %s.") \
             % self.connections_file
@@ -458,7 +462,7 @@ class GBaseApp:
                   ( category == option.category ) ):
           continue
 
-        allOptions [string.upper (option.longOption)] = option
+        allOptions [option.longOption.upper ()] = option
 
         if option.acceptsArgument:
           descr = '--%s <%s>' % (option.longOption, option.argumentName)
@@ -467,13 +471,11 @@ class GBaseApp:
         if option.shortOption:
           descr += ', -%s' % option.shortOption
 
-        descriptors [string.upper (option.longOption)] = descr
+        descriptors [option.longOption.upper()] = descr
 
-        if len (descr) > maxLength:
-          maxLength = len (descr)
+        maxLength = max(len (descr), maxLength)
 
-    if maxLength > 10:
-      maxLength = 10
+    maxLength = min(10, maxLength)
 
     sorted = allOptions.keys ()
     sorted.sort ()
@@ -491,7 +493,7 @@ class GBaseApp:
         dispOptions += "\n  %s  %s" % (descriptors [optionKey],
                " " * (maxLength - len (descriptors [optionKey])))
 
-      for word in string.split (allOptions [optionKey].help):
+      for word in allOptions [optionKey].help.split():
         if (len (word) + pos) > width:
           pos = 0
           dispOptions += "\n" + " " * margin
@@ -639,13 +641,13 @@ class GBaseApp:
 
     parameters = {}
     for param in paramList:
-      psplit = string.split (param, '=', 1)
+      psplit = param.split('=', 1)
       if len (psplit) == 1:
         raise StartupError, \
             'Parameter "%s" specified, but no value supplied.' % psplit [0]
-      parameters [string.lower (psplit [0])] = psplit [1]
+      parameters [psplit [0].lower()] = psplit [1]
 
-      gDebug (2,'Param "%s"="%s" ' % (string.lower (psplit [0]), psplit [1]))
+      assert gDebug (2,'Param "%s"="%s" ' % (psplit [0].lower(), psplit [1]))
 
     return parameters
 
@@ -664,11 +666,11 @@ class GBaseApp:
     self.printVersion ()
 
     # if msg is multiline, then surround with dashes to set it apart
-    if string.find ("%s" % msg, "\n") + 1:
+    if ("%s" % msg).find("\n") + 1:
       print '-' * 60
 
     print o(u_("Error: %s") % msg)
-    if string.find ("%s" % msg, "\n") + 1:
+    if ("%s" % msg).find("\n") + 1:
       print '-' * 60
 
     print o(u_("\nFor help, type:\n   %s --help\n") % self.COMMAND)
