@@ -17,7 +17,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: GConfig.py 7018 2005-02-12 23:29:37Z reinhard $
+# $Id: GConfig.py 7898 2005-09-12 14:31:40Z jamest $
 #
 # DESCRIPTION:
 # Class that loads the configuration files so gnue apps can get
@@ -67,7 +67,7 @@ class GConfig:
   #
   def loadApplicationConfig(self, configFilename="gnue.conf", homeConfigDir=".gnue", section="DEFAULT", defaults = None):
 
-    gDebug(2,'Reading configuration info from %s section %s' % (configFilename, section))
+    assert gDebug(2,'Reading configuration info from %s section %s' % (configFilename, section))
 
     #
     # Create parser and populate it if it doesn't exist
@@ -100,7 +100,7 @@ class GConfig:
       #
       try:
         parser.read(fileLocations)
-        gDebug(2,'Configuration files were read in this order:  %s' % \
+        assert gDebug(2,'Configuration files were read in this order:  %s' % \
                          (fileLocations) )
       except DuplicateSectionError:
         raise InvalidFormatError, _('The file has duplicate source definitions.')
