@@ -76,7 +76,7 @@ class Logger:
 
     for messageset in messages: 
       for message in string.split("%s" % messageset,'\n'): 
-        gDebug(2,'>> Logger: [%s] %s' % (timestamp, message))
+        assert gDebug(2,'>> Logger: [%s] %s' % (timestamp, message))
         try: 
           self._filehandle.write('[%s] %s\n' % (timestamp, message))
         except IOError, mesg:
