@@ -1,5 +1,8 @@
+# GNU Enterprise Common Library - Application Services
 #
-# This file is part of GNU Enterprise.
+# Copyright 2001-2005 Free Software Foundation
+#
+# This file is part of GNU Enterprise
 #
 # GNU Enterprise is free software; you can redistribute it
 # and/or modify it under the terms of the GNU General Public
@@ -16,33 +19,19 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# Copyright 2000-2005 Free Software Foundation
-#
-# FILE:
-# __init__.py
-#
-# DESCRIPTION:
+# $Id: __init__.py 7281 2005-03-31 21:39:54Z reinhard $
+
 """
+Basic services for all GNUe based applications
 """
-# NOTES:
-#
-
-
-# Python 2.2 introduced True/False types
-# For Python 2.0/2.1, we add them ourselves
-import sys
-if sys.hexversion < 0x02020000:
-  __builtins__['True']  = 1
-  __builtins__['False'] = 0
-
-# Add "basestring" for Python 2.2 or less
-if sys.hexversion < 0x02030000:
-  __builtins__['basestring'] = type("")
 
+# These modules introduce new functions into the builtins dictionary, so we
+# always import them
 import GDebug
 import i18n
-import checktype
 import errors
+import checktype
+
 
 # Add gStartupStatus global for startup status dialogs
 __builtins__['gStartupStatus'] = lambda string:string
