@@ -19,10 +19,15 @@
 #
 # Copyright 2001-2005 Free Software Foundation
 #
-# $Id: i18n.py 6976 2005-02-08 12:54:14Z reinhard $
+# $Id: i18n.py 7556 2005-05-31 22:09:38Z jan $
+
 """
-Internationalization support
+Internationalization support.
+
+@var language: Language of the current locale
+@var encoding: Encoding of the current locale
 """
+
 from types import *
 
 import exceptions
@@ -34,17 +39,31 @@ import sys
 
 from gnue import paths
 
+
 # -----------------------------------------------------------------------------
 # Global variables
 # -----------------------------------------------------------------------------
 
 __modules = {}                          # Modules by filename
-__catalogs = {}                         # Message catalogs by domain
-__userlanguage = None
+__catalogs = {}                         # Message catalogs by domain:language
 
 language = None
 encoding = None
-enc_policy = "replace"                  # policy to use if an unicode character
+
+
+# -----------------------------------------------------------------------------
+# Convert Unicode to String, let everything else untouched. This is o().
+# -----------------------------------------------------------------------------
+
+def outconv (message):
+  """
+  Encodes a message to L{encoding} (the current locale's encoding).  This
+  function is available as the builtin function "o()".
+  """
+  if isinstance (message, UnicodeType):
+    return message.encode (encoding, 'replace')
+  else:
+    return message
 
 
 # -----------------------------------------------------------------------------
@@ -64,6 +83,7 @@ def __find_module (filename):
         __modules [filename] = mod
         return mod
 
+
 # -----------------------------------------------------------------------------
 # Find the correct translation catalog
 # -----------------------------------------------------------------------------
@@ -84,22 +104,20 @@ def __find_catalog ():
   i = string.find (x, '.')
   domain = x [:i]
 
-  if __catalogs.has_key (domain):
-    return __catalogs [domain]
+  if __catalogs.has_key (domain + ':' + language):
+    return __catalogs [domain + ':' + language]
   else:
     try:
-      if os.name == 'posix':
-        catalog = gettext.translation (domain, paths.data + '/share/locale')
-      else:
-        catalog = gettext.translation (domain, paths.data + '/share/locale',
-                                       [language])
+      catalog = gettext.translation (domain, paths.data + '/share/locale',
+                                     [language])
     except:
       catalog = None
 
-    __catalogs [domain] = catalog
+    __catalogs [domain + ':' + language] = catalog
 
     return catalog
 
+
 # -----------------------------------------------------------------------------
 # Translate a message and return unicode
 # -----------------------------------------------------------------------------
@@ -116,55 +134,18 @@ def utranslate (message):
 
   return catalog.ugettext (message)
 
+
 # -----------------------------------------------------------------------------
 # Translate a message and return local encoding
 # -----------------------------------------------------------------------------
 
 def translate (message):
   """
-  Translates a message and returns an 8 bit string, encoded with @encoding (the
-  current locale's encoding).  This function is available as the builtin
+  Translates a message and returns an 8 bit string, encoded with L{encoding}
+  (the current locale's encoding).  This function is available as the builtin
   function "_()".
   """
-  catalog = __find_catalog ()
-
-  if catalog is None:
-    return message
-
-  return (catalog.ugettext (message)).encode (encoding, enc_policy)
-
-# -----------------------------------------------------------------------------
-# Convert Unicode to String, let everything else untouched. This is o().
-# -----------------------------------------------------------------------------
-
-def outconv (message):
-  """
-  Encodes a message to @encoding (the current locale's encoding).  This
-  function is available as the builtin function "o()".
-  """
-  if isinstance (message, UnicodeType):
-    return message.encode (encoding, enc_policy)
-  else:
-    return message
-
-
-# -----------------------------------------------------------------------------
-# Get the current encoding
-# -----------------------------------------------------------------------------
-
-def getencoding ():
-  """
-  This function returns the encoding of the currently active locale
-  @returns: encoding of the current locale
-  """
-  # For Windows, getlocale () is broken - it returns e.g. ('Hungarian_Hungary',
-  # '1250') instead of ('hu_HU', 'cp1250'). So we have to use getdefaultlocale.
-  # However, this means that under Windows, no locales but the default locale
-  # can be used.
-  if sys.platform == 'win32':
-    return locale.getdefaultlocale () [1] or 'ascii'
-  else:
-    return locale.getlocale () [1] or 'ascii'
+  return outconv (utranslate (message))
 
 
 # -----------------------------------------------------------------------------
@@ -173,20 +154,38 @@ def getencoding ():
 
 def getlanguage ():
   """
-  This function return the language of the currently acitve locale
-  @returns: language of the current locale
+  Returns the language of the currently acitve locale. This can be changed with
+  L{setcurrentlocale}.
+
+  @return: language of the current locale.
   """
-  return __userlanguage or 'C'
+  return language
 
 
-# ---------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
+# Get the current encoding
+# -----------------------------------------------------------------------------
+
+def getencoding ():
+  """
+  Returns the encoding of the currently active locale. This can be changed with
+  L{setcurrentlocale}.
+
+  @return: encoding of the current locale.
+  """
+  return encoding
+
+
+# -----------------------------------------------------------------------------
 # Get the locale string of the current user
-# ---------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 
 def getuserlocale ():
   """
-  This function tries to find out which locale the user is using.
-  @returns: localestring of the user's locale, i.e. de_AT@euro
+  Try to find out which locale the user is using.  This is always the locale of
+  the user running the program and is not touched by setcurrentlocale.
+
+  @return: localestring of the user's locale, i.e. de_AT@euro.
   """
 
   # *Actually* the following is very much what locale.getdefaultlocale should
@@ -210,38 +209,65 @@ def getuserlocale ():
   return result
 
 
-# ---------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
+# Update global variables
+# -----------------------------------------------------------------------------
+
+def __updateglobals ():
+
+  global language, encoding
+
+  # On win32, getlocale is broken - it returns strings like Hungarian_Hungary
+  # instead of hu_HU.
+  if sys.platform in ['win32','Pocket PC']:
+    (language, encoding) = locale.getdefaultlocale ()
+  else:
+    (language, encoding) = locale.getlocale ()
+
+  # Make sure language and encoding are not None
+  if not language:
+    language = 'C'
+  if not encoding:
+    encoding = 'ascii'
+
+
+# -----------------------------------------------------------------------------
 # Change the current locale
-# ---------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 
 def setcurrentlocale (newLocale):
   """
-  This function changes the current locale. If it fails it tries to succeed
-  using a more general form of the requested locale, i.e. if 'de_AT@euro'
-  fails, 'de_AT' will be tried next. If that fails too 'de' will be tried.
+  Set the current locale.
+  
+  If it fails it tries to succeed using a more general form of the requested
+  locale, i.e. if 'de_AT@euro' fails, 'de_AT' will be tried next. If that fails
+  too, 'de' will be tried.
+
   @param newLocale: string of the locale to be set, e.g. de_AT.ISO88591@euro
-      (full blown) or 'de_AT' or 'en_AU'
+    (full blown) or 'de_AT' or 'en_AU'
   """
-  if newLocale is None:
-    newLocale = ''
-
-  parts  = []
-  add    = []
-  normal = locale.normalize (newLocale)
-  next   = normal.split ('@') [0]
-  __userlanguage = next.split ('.') [0]
-
   # Setting a locale different than the current locale doesn't work on Windows.
   if sys.platform == 'win32':
     return
 
-  alias  = locale.locale_alias.get (__userlanguage.split ('_') [0].lower ())
+  if newLocale is None:
+    newLocale = ''
+
+  newLocale = newLocale.encode ()
+
+  parts  = []
+  add    = []
+  normal = locale.normalize (newLocale)         # lc_CC.ENCODING@variant
+  next   = normal.split ('@') [0]               # lc_CC.ENCODING
+  lang   = next.split ('.') [0]                 # lc_CC
+
+  alias  = locale.locale_alias.get (lang.split ('_') [0].lower ())
   if alias:
     add = [alias.split ('@') [0]]
     add.append (add [-1].split ('.') [0])
     add.append (locale.locale_alias.get (add [-1].split ('_') [0].lower ()))
 
-  for item in [normal, next, __userlanguage] + add:
+  for item in [normal, next, lang] + add:
     if item is not None and item not in parts:
       parts.append (item)
 
@@ -255,6 +281,8 @@ def setcurrentlocale (newLocale):
     else:
       break
 
+  __updateglobals ()
+
 
 # -----------------------------------------------------------------------------
 # Module initialization
@@ -266,20 +294,10 @@ try:
 except:
   pass
 
-# On win32, getlocale is broken - it returns Hungarian_Hungary instead of
-# hu_HU. On the other hand, getdefaultlocale is broken when $LANGUAGE contains
-# a ":" separated list of locales, like it does for Debian...
-if sys.platform == 'win32':
-  (language, encoding) = locale.getdefaultlocale ()
-else:
-  (language, encoding) = locale.getlocale ()
-
-# Make sure encoding is not None
-if not encoding:
-  encoding = 'ascii'
+__updateglobals ()
 
 # Now define the new builtin stuff
 import __builtin__
+__builtin__.__dict__['o'] = outconv
 __builtin__.__dict__['u_'] = utranslate
 __builtin__.__dict__['_'] = translate
-__builtin__.__dict__['o'] = outconv
