@@ -19,7 +19,7 @@
 #
 # Copyright 2001-2005 Free Software Foundation
 #
-# $Id: plugin.py 7007 2005-02-11 16:18:59Z reinhard $
+# $Id: plugin.py 7705 2005-07-07 15:16:13Z reinhard $
 
 """
 Functions to list and load avaliable plugins dynamically.
@@ -54,6 +54,7 @@ valid plugin.  Note that the module defining the __noplugin__ symbol is still
 imported, but no submodules of it. This is useful to exclude, for example,
 abstract base drivers.
 """
+
 from types import *
 
 import os
@@ -61,7 +62,8 @@ import string
 import sys
 import traceback
 
-from gnue.common.apps import i18n
+from gnue.common.apps import errors, i18n
+
 
 # =============================================================================
 # Exceptions
@@ -71,7 +73,7 @@ from gnue.common.apps import i18n
 # Module loading error
 # -----------------------------------------------------------------------------
 
-class LoadError (gException):
+class LoadError (errors.AdminError):
   """
   Indicates a failure to load a given module.  Raised by L{find}.
 
@@ -86,34 +88,34 @@ class LoadError (gException):
     self.exceptions = exceptions
 
     if self.exceptions:
-      message = u_("Cannot load plugin %s\n"
-                   "The following plugins failed:\n") % self.name
-      self.detail = message
-      for (name, exc) in exceptions.items ():
-        shortlist = traceback.format_exception_only (exc [0], exc [1])
-        del shortlist [1:2]
-        shortmsg = unicode (string.join (shortlist, ''), i18n.getencoding ())
-        longlist = traceback.format_exception (*exc)
-        del longlist [1:2]
-        longmsg = unicode (string.join (longlist, ''), i18n.getencoding ())
-        message += "* %s: %s" % (name, shortmsg)
-        self.detail += "* %s: %s" % (name, longmsg)
+      message = u_("Cannot load plugin '%s'") % self.name
+      detail = u_("The following plugins failed:\n")
+      for (name, exc) in self.exceptions.items ():
+        list = traceback.format_exception_only (exc [0], exc [1])
+        del list [1:2]
+        msg = unicode (string.join (list, ''), i18n.getencoding ())
+        detail += "* %s: %s" % (name, msg)
     else:
-      message = u_("Cannot find plugin %s") % self.name
-      self.detail = message
+      message = u_("Cannot find plugin '%s'") % self.name
+      detail = None
+
+    errors.AdminError.__init__ (self, message)
+
+    if detail:
+      self.detail = detail
 
-    gException.__init__ (self, message)
 
 # -----------------------------------------------------------------------------
 # List all available plugins
 # -----------------------------------------------------------------------------
 
-def list (base, identifier):
+def list (base, identifier, try_to_init = True):
   """
   List all available plugins.
 
   @param base: Name of the package that contains the plugins.
   @param identifier: Identifier that a plugin must define to qualify as module.
+  @param try_to_init: If set to False, __initplugin__ is not called.
   @return: A dictionary with the available plugin module names as keys and
       either the loaded module or the exception info tuple of the exception
       raised when trying to import the module as values.
@@ -127,7 +129,8 @@ def list (base, identifier):
   _identifier = identifier.encode ()
 
   # Now recursively list the plugins
-  return __list (_base, _identifier)
+  return __list (_base, _identifier, try_to_init, True)
+
 
 # -----------------------------------------------------------------------------
 # Find a plugin
@@ -167,6 +170,7 @@ def find (name, base, identifier):
   else:
     raise LoadError, (name, result)
 
+
 # -----------------------------------------------------------------------------
 # A list of all previously failed modules
 # -----------------------------------------------------------------------------
@@ -176,18 +180,28 @@ def find (name, base, identifier):
 # attempt.
 __failed = {}
 
+
 # -----------------------------------------------------------------------------
 # Mapping of previously found plugins
 # -----------------------------------------------------------------------------
+
 __findCache = {}
 
+
 # -----------------------------------------------------------------------------
 # Find all modules and subpackages in a package
 # -----------------------------------------------------------------------------
 
 def __modules (package, want_packages):
 
-  (basedir, basefile) = os.path.split (package.__file__)
+  # package.__file__ is a directory if GImportLogger is in use. This makes it
+  # necessary to 'simulate' a package, otherwise stepping down subpackages
+  # won't work.
+  if os.path.isdir (package.__file__):
+    (basedir, basefile) = (package.__file__, '__init__.py')
+  else:
+    (basedir, basefile) = os.path.split (package.__file__)
+
   (basename, baseext) = os.path.splitext (basefile)
 
   if basename != '__init__':
@@ -195,22 +209,27 @@ def __modules (package, want_packages):
     return []
 
   # Find all submodules
-  result = []
+  result = {}
   for subfile in os.listdir (basedir):
     (subname, subext) = os.path.splitext (subfile)
     subpath = os.path.join (basedir, subfile)
     # We are only interested in Python modules or packages
-    if (not want_packages and subext == '.py' and subname != '__init__') or \
+    if (not want_packages and subext in ['.py', '.pyc', '.pyo'] and \
+        subname != '__init__') or \
        (os.path.isdir (subpath) and \
-        os.path.isfile (os.path.join (subpath, '__init__.py'))):
-      result = result + [subname]
-  return result
+        os.path.isfile (os.path.join (subpath, '__init__.py')) or \
+        os.path.isfile (os.path.join (subpath, '__init__.pyc')) or \
+        os.path.isfile (os.path.join (subpath, '__init__.pyo'))):
+      result [subname] = True
+
+  return result.keys ()
+
 
 # -----------------------------------------------------------------------------
 # Recursively list all plugins
 # -----------------------------------------------------------------------------
 
-def __list (base, identifier):
+def __list (base, identifier, try_to_init, top):
 
   global __failed
 
@@ -228,22 +247,24 @@ def __list (base, identifier):
     # This is not a plugin, ignore it
     return {}
 
-  if hasattr (m, identifier):
-    # This is already a plugin, no need to go deeper
-    if hasattr (m, '__initplugin__'):
-      try:
-        m.__initplugin__ ()
-      except:
-        __failed [base] = sys.exc_info ()
-        return {base: __failed [base]}
-    return {base: m}
+  if not top:
+    if hasattr (m, identifier):
+      # This is already a plugin, no need to go deeper
+      if try_to_init and hasattr (m, '__initplugin__'):
+        try:
+          m.__initplugin__ ()
+        except:
+          __failed [base] = sys.exc_info ()
+          return {base: __failed [base]}
+      return {base: m}
 
   # List all submodules
   result = {}
   for sub in __modules (m, False):
-    result.update (__list (base + '.' + sub, identifier))
+    result.update (__list (base + '.' + sub, identifier, try_to_init, False))
   return result
 
+
 # -----------------------------------------------------------------------------
 # Recursively find first available plugin and return the module or a dictionary
 # with the exceptions that occured
@@ -286,6 +307,7 @@ def __first (base, identifier):
     exceptions.update (result)
   return exceptions
 
+
 # -----------------------------------------------------------------------------
 # Recursively search for a plugin and return the module or the exceptions that
 # occured
@@ -328,13 +350,15 @@ def __find (base, name, identifier):
 
   # Search all submodules
   exceptions = {}
-  for sub in __modules (m, True):
+  for sub in __modules (m, False):
     result = __find (base + '.' + sub, name, identifier)
     if isinstance (result, ModuleType):
       return result
     exceptions.update (result)
+
   return exceptions
 
+
 # =============================================================================
 # Self test code
 # =============================================================================
