@@ -81,7 +81,7 @@ class ColorScheme:
 
       self.__dict__['CP_%s'%attr] = (mod, pr)
       self.__dict__['C_%s'%attr] = v | mod
-      gDebug(5,'C_%s = %s' % (attr, v|mod))
+      assert gDebug(5,'C_%s = %s' % (attr, v|mod))
 
       pair += 1
 
