@@ -70,7 +70,7 @@ class FocusedLabel(Label):
         self.ExecMethod("SYSGOTFOCUS", None, None, None)
 
   def Paint(self, HasFocus, v2, v3):
-    gDebug(0,'Workin it')
+    assert gDebug(0,'Workin it')
     Container = self.PARENT.Screen()
     if self.active:
       if HasFocus :
