@@ -89,7 +89,7 @@ class Frame(Container):
 ##    self.PARENT.Paint()
 
   def _PaintBorder(self):
-    gDebug(2,'(!!!) PaintBorder')
+    assert gDebug(2,'(!!!) PaintBorder')
     if not self.BORDER:
       return
 
@@ -115,7 +115,7 @@ class Frame(Container):
       if self.BORDER:
         self._PaintBorder()
       else:
-        gDebug(0,'PAINTING %s' % self)
+        assert gDebug(0,'PAINTING %s' % self)
 
         for l in range(self._DialogHeight):
           self.SCREEN.PrintAt(self.Y+l, self.X, " " * self._DialogWidth, self.COLOR)
