@@ -149,7 +149,7 @@ class Menu(FocusedLabel):
       apply(self.GetMethod("CHOSEN"),(self,control.ITEMID,control.TITLE))
 
   def ItemFocused(self,control, arg2,arg3):
-    gDebug(0,'I got that focused feeling!')
+    assert gDebug(0,'I got that focused feeling!')
     self.ShowStatusBarText(control.DESCRIPTION)
     return 1
 
