@@ -36,7 +36,7 @@ class MenuBar(Frame):
   def __init__(self, Root, Y, X, W,  **properties):
     Frame.__init__(self, Root, Y, X, Y+1, X + W - 1,
                    border=0, dropshadow=0, **properties)
-    gDebug(0,'WIDTH=%s' % W)
+    assert gDebug(0,'WIDTH=%s' % W)
     self.rootwin = Root
     self.CANGETFOCUS = 0
     self.Y = Y
@@ -55,7 +55,7 @@ class MenuBar(Frame):
     return newEntry
 
   def SetStatusBar(self, statusbar, index=0):
-    gDebug(0,'SETTING STATUSBAR')
+    assert gDebug(0,'SETTING STATUSBAR')
     self.STATUSBAR = statusbar
     self.STATUSBAR_INDEX = index
 
