@@ -111,7 +111,7 @@ class StatusBar(Control):
     self.SetMethod("SYSGOTFOCUS", None)
     self.SetMethod("SYSLOSTFOCUS", None)
 
-    gDebug(1,"X=%s,Y=%s,W=%s, Xs=%s, lenEmpty=%s, Sizes=%s" % \
+    assert gDebug(1,"X=%s,Y=%s,W=%s, Xs=%s, lenEmpty=%s, Sizes=%s" % \
                        (X, Y, W, self.FIELD_X, len(self._EmptyStatusBar), Sizes))
 
 
