@@ -97,20 +97,20 @@ class _Sio:
 
   def BeginScreenAccess(self):
     """Initialize the screen and setup for action."""
-    gDebug(0,'initializing curses screen')
+    assert gDebug(0,'initializing curses screen')
     #if self.s <> None:
     #  return
-    gDebug(0,'initializing curses screen 1')
+    assert gDebug(0,'initializing curses screen 1')
     self.stdscr = curses.initscr()
     self._NegotiateScreenSize()
-    gDebug(0,'initializing curses screen 2')
+    assert gDebug(0,'initializing curses screen 2')
     self.s = curses.newwin(self.MAXROW+1, self.MAXCOL+1, 0, 0)
     curses.noecho()
-    gDebug(0,'initializing curses screen 3')
+    assert gDebug(0,'initializing curses screen 3')
     #curses.nonl()  # disallows usage of <CR> as a selector.
     curses.cbreak()
     self.s.keypad(1)
-    gDebug(0,'initializing curses screen 4')
+    assert gDebug(0,'initializing curses screen 4')
     # make the cursor very visible
     #curses.curs_set(2) # throws an ERR, removing.
 
@@ -128,7 +128,7 @@ class _Sio:
       tiLTEE : curses.ACS_LTEE,
       tiRTEE : curses.ACS_RTEE,
       tiBULL : curses.ACS_PLUS}
-    gDebug(0,'initializing curses screen 5')
+    assert gDebug(0,'initializing curses screen 5')
 
     # TODO: should really only do this if configuration
     # TODO: says yes, or on some other test.
@@ -137,7 +137,7 @@ class _Sio:
     import ColorScheme
 
     if curses.has_colors():
-      gDebug(1,"Initializing StandardWhiteOnBlue colors")
+      assert gDebug(1,"Initializing StandardWhiteOnBlue colors")
       self.colors = ColorScheme.StandardWhiteOnBlue()
 
 #        curses.init_pair(C_SCREEN, curses.COLOR_WHITE,curses.COLOR_BLUE)
@@ -160,7 +160,7 @@ class _Sio:
 ##        curses.init_pair(C_STATUSBAR, curses.COLOR_BLACK, curses.COLOR_WHITE)
 ##        curses.init_pair(C_STATUSBAR_FIELD, curses.COLOR_BLUE, curses.COLOR_WHITE)
     else:
-      gDebug(1,"Initializing Monochrome colors")
+      assert gDebug(1,"Initializing Monochrome colors")
       self.colors = ColorScheme.Monochrome()
 
 
