@@ -19,7 +19,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: GConnections.py 7810 2005-08-09 18:28:25Z reinhard $
+# $Id: GConnections.py 7916 2005-09-19 09:44:26Z johannes $
 """
 Connection manager system.
 """
@@ -209,7 +209,7 @@ class GConnections:
       except:
         return default
     except KeyError:
-      raise NotFoundError, connection_name, self._location
+      raise NotFoundError, (connection_name, self._location)
 
 
   # ---------------------------------------------------------------------------
@@ -263,7 +263,7 @@ class GConnections:
     try:
       return copy.deepcopy (self.__definitions [connection_name])
     except KeyError:
-      raise NotFoundError, connection_name, self._location
+      raise NotFoundError, (connection_name, self._location)
 
 
   # ---------------------------------------------------------------------------
@@ -392,11 +392,11 @@ class GConnections:
         netrcData = netrc.netrc ().authenticators ("'gnue://%s/'" \
                                                    % connection_base)
         if netrcData is not None:
-          gDebug (7, 'Read the user\'s .netrc file')
+          assert gDebug (7, 'Read the user\'s .netrc file')
           loginData ['_username'] = netrcData [0][1:-1]
           loginData ['_password'] = netrcData [2][1:-1]
 
-          gDebug (7, "Found useful stuff for connection %s in "
+          assert gDebug (7, "Found useful stuff for connection %s in "
                                "the user\'s .netrc file" % connection_name)
 
       except (IOError, netrc.NetrcParseError, KeyError):
@@ -466,7 +466,7 @@ class GConnections:
 
         attempts = 4
 
-        gDebug (7, 'Getting new data connection to %s' % connection_name)
+        assert gDebug (7, 'Getting new data connection to %s' % connection_name)
 
         errortext = None
 
@@ -532,32 +532,6 @@ class GConnections:
       return None
 
 
-  # ---------------------------------------------------------------------------
-  # Committ all connections
-  # ---------------------------------------------------------------------------
-
-  def commitAll (self):
-    """
-    This function commits all transactions for all open connections.
-    """
-
-    for connection in self.__openConnections.values():
-      connection.commit()
-
-
-  # ---------------------------------------------------------------------------
-  # Rollback all connections
-  # ---------------------------------------------------------------------------
-
-  def rollbackAll(self):
-    """
-    This function rolls back all transactions for all open connections.
-    """
-
-    for connection in self.__openConnections.values():
-      connection.rollback()
-
-
   # ---------------------------------------------------------------------------
   # Close all connections
   # ---------------------------------------------------------------------------
@@ -567,11 +541,20 @@ class GConnections:
     This function closes all open connections.
     """
 
-    for connection in self.__openConnections.values():
+    for connection in self.__openConnections.values ():
       connection.close ()
 
-      if self.__authenticatedUsers.has_key (connection):
-        del self.__authenticatedUsers [connection]
 
-    for k in self.__openConnections.keys ():
-      del self.__openConnections [k]
+
+  # ---------------------------------------------------------------------------
+  # After a connection has been closed, remove all references to it
+  # ---------------------------------------------------------------------------
+
+  def _connectionClosed (self, connection):
+
+    if connection in self.__authenticatedUsers:
+      del self.__authenticatedUsers [connection]
+
+    for (key, value) in self.__openConnections.items ():
+      if value == connection:
+        del self.__openConnections [key]
