@@ -19,7 +19,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: GDataSource.py 7827 2005-08-12 13:18:37Z reinhard $
+# $Id: GDataSource.py 7909 2005-09-15 12:20:55Z reinhard $
 """
 Classes for the datasource object tree.
 """
@@ -200,15 +200,20 @@ class GDataSource (GObjects.GObj):
   # ---------------------------------------------------------------------------
 
   def __trigger_call (self, name, params):
-    # The update function is not available for backends that need a rollback
+    # The call function is not available for backends that need a rollback
     # after a post have failed. Reason: we do an uncommmitted post here, and a
     # rollback in a later post would also roll back this post, and all changes
     # in this post would get lost.
     if self._connection._need_rollback_after_exception_:
       raise Exceptions.FunctionNotAvailable
+    # Remember current record; record pointer is not reliable between postAll
+    # and requeryAll!
+    current = self.__currentResultSet.current
     self.postAll ()
-    result = self.__currentResultSet.current.call (name, params)
-    self.requeryAll (False)
+    try:
+      result = current.call (name, params)
+    finally:
+      self.requeryAll (False)
     return result
 
   # ---------------------------------------------------------------------------
@@ -491,7 +496,7 @@ class GDataSource (GObjects.GObj):
     """
 
     if self.type == 'unbound':
-      gDebug (1, "Trying to bind field %s to unbound DataSource" % field)
+      assert gDebug (1, "Trying to bind field %s to unbound DataSource" % field)
       return
 
     self.__fieldReferences [field] = True
@@ -537,7 +542,7 @@ class GDataSource (GObjects.GObj):
     @param defaultValue: optional default value for newly inserted records.
     """
 
-    gDebug (7,'Unbound Field %s implicitly referenced' % field)
+    assert gDebug (7,'Unbound Field %s implicitly referenced' % field)
 
     if defaultValue != None:
       self.__defaultData [field] = defaultValue
@@ -594,9 +599,17 @@ class GDataSource (GObjects.GObj):
       merged.
     """
 
-    newResultSet = self.__createResultSet (masterRecord = masterRecord)
-    resultSet._merge (newResultSet)
-    newResultSet.close ()
+    if self.__rowidField or self.__primarykeyFields:
+      newResultSet = self.__createResultSet (masterRecord = masterRecord)
+      resultSet._merge (newResultSet)
+      newResultSet.close ()
+    else:
+      # We can't merge the resultsets without a key, so no use requerying at
+      # all. The call to resultSet.requery will just reset the status for all
+      # records and remove the deleted records from the cache.
+      # We can pretend there has been a commit in any case: The requery isn't
+      # done against the database anyway.
+      resultSet.requery (True)
 
     # If this is the current result set, then the UI has to follow the changes.
     if resultSet == self.__currentResultSet:
@@ -733,7 +746,7 @@ class GDataSource (GObjects.GObj):
         else:
           # we don't have to rollback, the changes done so far to the backend
           # are preserved, we requery so we have the the frontend updated
-          self.__currentResultSet.requery ()
+          self.__currentResultSet.requery (False)
         raise
 
 
@@ -778,7 +791,7 @@ class GDataSource (GObjects.GObj):
     # If it's a string or a unicode string, we transform it into a tuple
     # sequence, where all items are treated to be in 'ascending' order
     if isinstance (order_by, basestring):
-      gDebug (1, "DEPRECIATION WARNING: use of 'order_by' attribute is " \
+      assert gDebug (1, "DEPRECIATION WARNING: use of 'order_by' attribute is " \
                  "depreciated. Please use <sortorder> instead.")
       for field in order_by.split (','):
         (item, desc) = (field, field [-5:].lower () == ' desc')
