@@ -19,7 +19,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: RecordSet.py 7827 2005-08-12 13:18:37Z reinhard $
+# $Id: RecordSet.py 7951 2005-09-21 15:37:49Z reinhard $
 
 """
 RecordSet class used by all database driver plugins.
@@ -137,9 +137,9 @@ class RecordSet:
     self.__eventController  = eventController
 
     # Record status
-    # New recs:      'empty' -(__setitem__)-> 'inserted' -(delete)-> 'void'
-    # Existing recs: 'clean' -(__setitem__)-> 'modified' -(delete)-> 'deleted'
-    self.__status = 'clean'
+    self.__inserted = False             # True = new record, False = existing
+    self.__modified = False             # True = dirty, False = clean
+    self.__deleted = False              # True = deleted, False = still alive
 
     # The field values
     self.__fields = {}
@@ -168,19 +168,21 @@ class RecordSet:
     #            requeried again after the following commit
     self.__requeryStatus = None
 
+    # While this is true, changes don't mark the record as dirty.
+    self.__initializing = True
+
     if self.__initialData:
 
       # Existing record:
       # Set the current state of all fields as given in the parameter
       self.__fields = self.__initialData.copy ()
-      if self.__eventController is not None:
-        self.__eventController.dispatchEvent ('dsRecordLoaded')
+      self.__dispatchEvent ('dsRecordLoaded')
 
     else:
 
       # New record:
       # 1. mark as new
-      self.__status = 'empty'
+      self.__inserted = True
 
       # 2. Get the default values from the backend
       if self.__connection:
@@ -198,7 +200,12 @@ class RecordSet:
 
       # 4. Get default values from DataSource (includes link to master).
       for (fieldname, value) in defaultData.items ():
-        self.__setField (fieldname, value)
+        self [fieldname] = value
+
+      # 5. Notify event listener about new record
+      self.__dispatchEvent ('dsRecordInserted')
+
+    self.__initializing = False
 
 
   # ---------------------------------------------------------------------------
@@ -245,15 +252,12 @@ class RecordSet:
     """
     if fieldname in self.__boundFields and self.__readonly:
       raise Exceptions.ReadOnlyModifyError
-    self.__setField (fieldname, value)
-    if fieldname in self.__boundFields:
-      if self.__status in ['empty', 'clean']:
-        if self.__status == 'empty':
-          self.__status = 'inserted'
-        elif self.__status == 'clean':
-          self.__status = 'modified'
-        if self.__eventController is not None:
-          self.__eventController.dispatchEvent ('dsRecordTouched')
+    self.__fields [fieldname] = value
+    self.__modifiedFlags [fieldname] = True
+    if not self.__initializing and fieldname in self.__boundFields:
+      if not self.__modified:
+        self.__modified = True
+        self.__dispatchEvent ('dsRecordTouched')
 
   # ---------------------------------------------------------------------------
 
@@ -305,21 +309,6 @@ class RecordSet:
       self [fieldname] = value
 
 
-  # ---------------------------------------------------------------------------
-  # Set a field and mark it as dirty, leaving the record state unchanged
-  # ---------------------------------------------------------------------------
-
-  def __setField (self, fieldname, value):
-
-    # Calling this function directly (instead of __setitem__) causes the field
-    # to be marked as dirty, but the complete record remains unchanged. This is
-    # useful to initialize a new record: the record will still be regarded as
-    # "empty", but the field will be included in any following insert
-    # statement.
-    self.__fields [fieldname] = value
-    self.__modifiedFlags [fieldname] = True
-
-
   # ---------------------------------------------------------------------------
   # Find out if a field is modified
   # ---------------------------------------------------------------------------
@@ -361,12 +350,30 @@ class RecordSet:
     if self.__readonly:
       raise Exceptions.ReadOnlyDeleteError
 
-    if self.__status in ['empty', 'inserted']:
-      self.__status = 'void'
-    elif self.__status in ['clean', 'modified']:
-      self.__status = 'deleted'
+    self.__deleted = True
 
 
+  # ---------------------------------------------------------------------------
+  # Remove the delete mark from the record
+  # ---------------------------------------------------------------------------
+
+  def undelete (self):
+    """
+    Remove the delete mark from the record
+
+    The record is restored to the status it had prior to being marked for 
+    delete.
+
+    @raise Exceptions.ReadOnlyDeleteError: if the RecordSet is read only.
+    """
+
+    if self.__readonly:
+      raise Exceptions.ReadOnlyDeleteError
+
+    assert self.__deleted
+    self.__deleted = False
+  
+
   # ---------------------------------------------------------------------------
   # Call backend code
   # ---------------------------------------------------------------------------
@@ -387,7 +394,7 @@ class RecordSet:
     @param methodname: Name of the function to call.
     @param parameters: Dictionary with parametername/value pairs.
     @return: Return value of the function that was called.
-    @raise Exceptions.FunctionCallOnEmptyRecordError: if the record is
+    @raise Exceptions.FunctionCallOfEmptyRecordError: if the record is
       empty.
     @raise Exception: whatever the called function raises
     """
@@ -396,7 +403,7 @@ class RecordSet:
     checktype (parameters, dict)
 
     if self.isEmpty ():
-      raise Exceptions.FunctionCallOnEmptyRecordError
+      raise Exceptions.FunctionCallOfEmptyRecordError
 
     return self.__connection.call (self.__tablename, self.__wherefields (),
         methodname, parameters)
@@ -414,7 +421,16 @@ class RecordSet:
     been changed nor has a detail for this record been inserted with a status
     other than empty.
     """
-    return self.__status == 'empty' and not self.__hasPendingChildren ()
+    return self.__inserted and not self.__modified and not self.__deleted \
+        and not self.__hasPendingChildren ()
+
+  # ---------------------------------------------------------------------------
+
+  def isVoid (self):
+    """
+    Return True if the record has been inserted and then deleted.
+    """
+    return self.__inserted and self.__deleted
 
   # ---------------------------------------------------------------------------
 
@@ -425,8 +441,9 @@ class RecordSet:
 
     Records with this status will be inserted into the database on post.
     """
-    return self.__status == 'inserted' or \
-        (self.__status == 'empty' and self.__hasPendingChildren ())
+    return self.__inserted \
+        and (self.__modified or self.__hasPendingChildren ()) \
+        and not self.__deleted
 
   # ---------------------------------------------------------------------------
 
@@ -436,8 +453,9 @@ class RecordSet:
 
     Records with this status will be updated in the database on post.
     """
-    return self.__status == 'modified' or \
-        (self.__status == 'clean' and self.__hasPendingChildren ())
+    return not self.__inserted \
+        and (self.__modified or self.__hasPendingChildren ()) \
+        and not self.__deleted
 
   # ---------------------------------------------------------------------------
 
@@ -447,7 +465,7 @@ class RecordSet:
 
     Records with this status will be deleted in the database on post.
     """
-    return self.__status == 'deleted'
+    return not self.__inserted and self.__deleted
 
   # ---------------------------------------------------------------------------
 
@@ -457,11 +475,14 @@ class RecordSet:
     post it to the database.
 
     The result is True if either isInserted, isModified, or isDeleted is True.
+
+    If the record is void (i.e. has been inserted and then deleted again),
+    isPending also returns True even though no changes will be written to the
+    backend.
     """
-    # ... but we check __status instead of isXxxx to not have to evaluate
+    # ... but we check status fields instead of isXxxx to not have to evaluate
     # __hasPendingChildren several times.
-    return self.__status in ['inserted', 'modified', 'deleted'] or \
-        (self.__status in ['empty', 'clean'] and self.__hasPendingChildren ())
+    return (self.__modified or self.__hasPendingChildren ()) or self.__deleted
 
   # ---------------------------------------------------------------------------
 
@@ -484,7 +505,8 @@ class RecordSet:
       happened has been committed.
     """
     return (self.__requeryStatus == 'posted') \
-        or (self.__requeryStatus == 'commit' and commit)
+        or (self.__requeryStatus == 'commit' and commit) \
+        or self.isEmpty () or self.isVoid ()
 
 
   # ---------------------------------------------------------------------------
@@ -506,20 +528,17 @@ class RecordSet:
     if self.__requeryStatus == 'posted':
       # record has been written to the backend - everything is clean now
       self.__fields.update (data)
-      self.__status = 'clean'
+      self.__inserted = False
+      self.__modified = False
       self.__modifiedFlags = {}
       self.__requeryStatus = None
 
-    elif self.__status == 'clean':
-      # record is clean anyway
-      self.__fields.update (data)
-
     else:
-      # record has unsaved changes because the last _post to this record (or a
-      # preceding record) failed - we have to be cautious not to overwrite
-      # changes the user has done
-      for (fieldname, value) in data:
-        if not self.__modifiedFlags [fieldname]:
+      # record may have unsaved changes because the last _post to this record
+      # (or a preceding record) failed - we have to be cautious not to
+      # overwrite changes the user has done
+      for (fieldname, value) in data.items ():
+        if not self.__modifiedFlags.has_key (fieldname):
           self.__fields [fieldname] = value
 
 
@@ -544,9 +563,9 @@ class RecordSet:
           dataSource._activateResultSet (resultset)
           continue
         
-      # If this record is empty anyway, it can't have any detail records, so
+      # If this record is new, it can't have any detail records yet anyway, so
       # create empty detail result sets.  Query the matching details otherwise.
-      if self.__status == 'empty':
+      if self.__inserted:
         resultset = dataSource.createEmptyResultSet (masterRecord = self)
       else:
         resultset = dataSource.createResultSet (masterRecord = self)
@@ -575,25 +594,21 @@ class RecordSet:
     """
 
     # Just to make sure - you never know who calls us...
-    if not self.isPending ():
+    if not self.isPending () or self.isVoid ():
       return
 
-    # Save the initial status so we know if any triggers changed us
-    status = self.__status
-
     # Call the hooks for commit-level hooks
-    if self.__eventController is not None:
-      # A trigger code could change the status from empty/inserted/modified to
-      # deleted. In that case, both triggers would be called.
-      if self.__status in ['empty', 'inserted']:
-        self.__eventController.dispatchEvent ('dsCommitInsert')
-      if self.__status == 'modified':
-        self.__eventController.dispatchEvent ('dsCommitUpdate')
-      if self.__status == 'deleted':
-        self.__eventController.dispatchEvent ('dsCommitDelete')
+    # A trigger code could change the status from empty/inserted/modified to
+    # deleted. In that case, both triggers would be called.
+    if self.__inserted:
+      self.__dispatchEvent ('dsCommitInsert')
+    elif self.__modified:
+      self.__dispatchEvent ('dsCommitUpdate')
+    if self.__deleted:
+      self.__dispatchEvent ('dsCommitDelete')
 
     # Check for empty primary key and set with the sequence value if so
-    if self.__status in ['empty', 'inserted']:
+    if self.__inserted:
       if len (self.__primarykeyFields) == 1 and \
           self [self.__primarykeyFields [0]] is None and \
           self.__primarykeySeq is not None and \
@@ -603,15 +618,15 @@ class RecordSet:
 
     # If we have a connection (i.e. we aren't static or unbound), do the post
     if self.__connection is not None:
-      if self.__status == 'deleted':
+      if self.__deleted:
         self.__connection.delete (self.__tablename, self.__wherefields ())
-      elif self.__status in ['empty', 'inserted', 'modified']:
+      elif self.__inserted or self.__modified:
         modifiedFields = {}
         for field in self.__boundFields:
           if self.__modifiedFlags.has_key (field):
             modifiedFields [field] = self.__fields [field]
 
-        if self.__status in ['empty', 'inserted']:
+        if self.__inserted:
           rowid = self.__connection.insert (self.__tablename, modifiedFields)
           if self.__rowidField:
             self.__fields [self.__rowidField] = rowid
@@ -656,7 +671,8 @@ class RecordSet:
       return
 
     # The record is now "clean" again
-    self.__status = 'clean'
+    self.__inserted = False
+    self.__modified = False
     self.__modifiedFlags = {}
     self.__initialData = self.__fields.copy ()
     if commit:
@@ -711,6 +727,15 @@ class RecordSet:
     return result
 
 
+  # ---------------------------------------------------------------------------
+  # Dispatch an event
+  # ---------------------------------------------------------------------------
+
+  def __dispatchEvent (self, event):
+    if self.__eventController is not None:
+      self.__eventController.dispatchEvent (event, record = self)
+
+
   # ---------------------------------------------------------------------------
   # Depreciated methods (replaced by dictionary functions)
   # ---------------------------------------------------------------------------
