@@ -1,3 +1,6 @@
+# GNU Enterprise Common Library - Base database driver
+#
+# Copyright 2000-2005 Free Software Foundation
 #
 # This file is part of GNU Enterprise.
 #
@@ -16,21 +19,16 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# Copyright 2000-2005 Free Software Foundation
-#
-# FILE:
-# Base/__init__.py
-#
-# DESCRIPTION:
-#
-# NOTES:
-#
-# $Id :$
+# $Id: __init__.py 7631 2005-06-20 15:43:48Z reinhard $
+
+"""
+Generic base classes for database driver plugins.
+"""
 
 # Indicate that this is no plugin
 __noplugin__ = True
 
 from Connection import Connection
-from DataObject import DataObject
 from ResultSet import ResultSet
 from RecordSet import RecordSet
+from Behavior import Behavior
