@@ -19,7 +19,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: Connection.py 7785 2005-08-05 10:02:54Z reinhard $
+# $Id: Connection.py 7980 2005-09-24 05:50:21Z reinhard $
 
 """
 Generic Connection class for DBSIG2 based database driver plugins.
@@ -112,7 +112,7 @@ class Connection (Base.Connection):
 
   def _connect_ (self, connectData):
     (params, kwargs) = self._getConnectParams_ (connectData)
-    gDebug (3, 'DBSIG2 Connect')
+    assert gDebug (3, 'DBSIG2 Connect')
     try:
       self._native = self._driver.connect (*params, **kwargs)
     except self._driver.DatabaseError:
@@ -131,7 +131,7 @@ class Connection (Base.Connection):
       parameters [key] = value
     statement = "INSERT INTO %s (%s) VALUES (%s)" % (table, ', '.join (fields),
         ', '.join (values))
-    return self.__execute (statement, parameters)
+    return self.sql0 (statement, parameters)
 
   # ---------------------------------------------------------------------------
 
@@ -144,14 +144,14 @@ class Connection (Base.Connection):
       parameters [key] = value
     statement = "UPDATE %s SET %s WHERE %s" % (table, ', '.join (updates),
         where)
-    self.__execute (statement, parameters)
+    self.sql0 (statement, parameters)
 
   # ---------------------------------------------------------------------------
 
   def _delete_ (self, table, oldfields):
     (where, parameters) = self.__where (oldfields)
     statement = 'DELETE FROM %s WHERE %s' % (table, where)
-    self.__execute (statement, parameters)
+    self.sql0 (statement, parameters)
 
   # ---------------------------------------------------------------------------
 
@@ -159,10 +159,7 @@ class Connection (Base.Connection):
     (where, parameters) = self.__where (oldfields)
     statement = "SELECT %s FROM %s WHERE %s" % (', '.join (fields), table,
         where)
-    try:
-      rows = self.sql (statement, parameters)
-    except self._driver.DatabaseError:
-      raise Exceptions.ConnectionError, errors.getException () [2]
+    rows = self.sql (statement, parameters)
     if len (rows):
       row = rows [0]
       result = {}
@@ -178,23 +175,24 @@ class Connection (Base.Connection):
   # ---------------------------------------------------------------------------
 
   def _commit_ (self):
-    gDebug (3, 'DBSIG2 Commit')
+    assert gDebug (3, 'DBSIG2 Commit')
     try:
       self._native.commit ()
     except self._driver.DatabaseError:
-      raise Exceptions.ConnectionError, errors.getException () [2]
+      raise Exceptions.ConnectionError, (errors.getException () [2], 'COMMIT',
+          None)
 
   # ---------------------------------------------------------------------------
 
   def _rollback_ (self):
-    gDebug (3, 'DBSIG2 Rollback')
+    assert gDebug (3, 'DBSIG2 Rollback')
     if hasattr (self._native, 'rollback'):
       self._native.rollback ()
 
   # ---------------------------------------------------------------------------
 
   def _close_ (self):
-    gDebug (3, 'DBSIG2 Close')
+    assert gDebug (3, 'DBSIG2 Close')
     if self._native:
       self._native.close ()
 
@@ -218,18 +216,6 @@ class Connection (Base.Connection):
     return (' AND '.join (where), parameters)
 
 
-  # ---------------------------------------------------------------------------
-  # Execute an insert, update, or delete statement
-  # ---------------------------------------------------------------------------
-
-  def __execute (self, statement, parameters):
-
-    try:
-      return self.sql0 (statement, parameters)
-    except self._driver.DatabaseError:
-      raise Exceptions.ConnectionError, errors.getException () [2]
-
-
   # ---------------------------------------------------------------------------
   # Execute the given SQL statement and return the result matrix
   # ---------------------------------------------------------------------------
@@ -358,8 +344,8 @@ class Connection (Base.Connection):
     else:
       p = None
 
-    gDebug (3, "DBSIG2 Statement: %s" % s)
-    gDebug (3, "DBSIG2 Parameters: %s" % p)
+    assert gDebug (3, "DBSIG2 Statement: %s" % s)
+    assert gDebug (3, "DBSIG2 Parameters: %s" % p)
 
     # Create DBSIG2 cursor and execute statement
     cursor = self._native.cursor ()
@@ -370,7 +356,8 @@ class Connection (Base.Connection):
         cursor.execute (s)
     except:
       cursor.close ()
-      raise
+      raise Exceptions.ConnectionError, (errors.getException () [2], statement,
+          parameters)
 
     return cursor
 
