@@ -19,7 +19,7 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: appserver.py 7791 2005-08-06 19:14:11Z reinhard $
+# $Id: appserver.py 8001 2005-09-27 04:12:41Z johannes $
 
 """
 Database driver plugin for GNU Enterprise Application Server backends.
@@ -313,7 +313,6 @@ class Connection (Base.Connection):
   _resultSetClass_                = ResultSet
   _behavior_                      = Behavior
   _primarykeyFields_              = [u'gnue_id']
-  _need_rollback_after_exception_ = False
 
 
   # ---------------------------------------------------------------------------
@@ -324,7 +323,6 @@ class Connection (Base.Connection):
 
     Base.Connection.__init__ (self, connections, name, parameters)
     self.__filters = None
-    self.__server  = None
     self._sm       = None
 
 
@@ -493,6 +491,8 @@ class Connection (Base.Connection):
   def _close_ (self):
     if self._sm is not None:
       self._sm.close (self._sess_id, False)
+      self._sm._destroy ()
+      self._sm = None
 
 
   # ---------------------------------------------------------------------------
@@ -501,17 +501,14 @@ class Connection (Base.Connection):
 
   def __getSessionManager (self):
 
-    if self.__server is None:
+    if self._sm is None:
       params = {'host'     : self.parameters.get ('host'),
                 'port'     : self.parameters.get ('port'),
                 'transport': self.parameters.get ('transport')}
+      if 'timeout' in self.parameters:
+        params ['timeout'] = float (self.parameters ['timeout'])
+
       rpcType = self.parameters.get ('rpctype')
-      self.__server = client.attach (rpcType, params)
-
-      if self.parameters.has_key ('timeout'):
-        self.__server.setTimeout (float (self.parameters ['timeout']))
-
-    if self._sm is None:
-      self._sm = self.__server.request ('Session')
+      self._sm = client.attach (rpcType, params)
 
     return self._sm
