@@ -162,7 +162,7 @@ def loadXMLObject(stream, handler, rootType, xmlFileType,
   object.walk(addAttributesWalker,attributes=attributes)
 
   if initialize:
-    gDebug (7, "Initializing the object tree starting at %s" %(object))
+    assert gDebug (7, "Initializing the object tree starting at %s" %(object))
     object.phaseInit(dh._phaseInitCount)
 
   # Break the cyclic reference in the content handler, so garbage collection is
@@ -296,7 +296,7 @@ class xmlHandler(xml.sax.ContentHandler):
       #
       # No namespace qualifier
       #
-      gDebug (7, "<%s>" % name)
+      assert gDebug (7, "<%s>" % name)
 
       try:
         baseAttrs = self.xmlElements[name].get('Attributes',{})
@@ -400,7 +400,7 @@ class xmlHandler(xml.sax.ContentHandler):
       # namespace qualifier and we are masquerading
       #
 
-      gDebug (7, "<%s:%s>" % (ns,name))
+      assert gDebug (7, "<%s:%s>" % (ns,name))
 
       for qattr in saxattrs.keys():
         attrns, attr = qattr
@@ -504,7 +504,7 @@ class xmlHandler(xml.sax.ContentHandler):
     inits = child._buildObject()
     self._phaseInitCount = (inits != None and inits > self._phaseInitCount \
                             and inits or self._phaseInitCount)
-    gDebug (7, "</%s>" % name)
+    assert gDebug (7, "</%s>" % name)
 
 
 
@@ -557,7 +557,7 @@ class GImportItem(GObj):
        for key in self._loadedxmlattrs.keys():
          if key[0] != '_':
            rv.__dict__[key] = self._loadedxmlattrs[key]
-           gDebug (7, ">>> Moving %s" % key)
+           assert gDebug (7, ">>> Moving %s" % key)
        rv._buildObject()
      else:
          raise MarkupError, \
