@@ -44,7 +44,7 @@ def applyFormatting (value, mask):
     # TODO: This whole section should be using FormatMasks
     # TODO: This is all a gross hack!!!!
     try:
-      return value.strftime(mask)
+      return value.strftime(str(mask))
     except AttributeError:
       pass
 
