@@ -74,7 +74,6 @@ class GObjNamespace(GObj):
   #
   def constructTriggerObject(self, gobjObject, triggerParent=None):
     triggerObject = None
-
     # Some items in a GObj tree may not be GObj based (GContent for instance)
 
     if isinstance(gobjObject,GObj) and hasattr(gobjObject,'name'):
@@ -117,14 +116,7 @@ class GObjNamespace(GObj):
 
       # Process the children of this Gobj
       if len(gobjObject._children):
-        for child in gobjObject._children:
-          object = self.constructTriggerObject(child, triggerObject)
-
-          # Add this objects children to it's namespace by their name
-          if object:
-            if child.name is not None:
-              if child.name [:2] != '__':
-                triggerObject.__dict__[child.name] = object
+        self.addChildNames(gobjObject._children, triggerObject)
 
       #
       # populate the GObj's _localTriggerNamespace
@@ -134,6 +126,21 @@ class GObjNamespace(GObj):
 
     return triggerObject
 
+  def addChildNames(self, children, triggerObject):
+        for child in children:
+          object = self.constructTriggerObject(child, triggerObject)
+
+          # Add this objects children to it's namespace by their name
+          if object:
+            if child.name is not None:
+              # skip on GRPassThru objects
+              if child.name [:2] != '__':
+                triggerObject.__dict__[child.name] = object
+              else:
+                if len(child._children):
+                  self.addChildNames(child._children, triggerObject)
+    
+
 #
 # NamespaceElement
 #
@@ -142,6 +149,8 @@ class GObjNamespace(GObj):
 class NamespaceElement(GObj):
   def __init__(self, parent):
     GObj.__init__(self,parent)
+    # TODO: Check wether this _parent attribute is really needed, since it
+    # should be a weak reference (or replaced by getParent ())
     self.__dict__['_parent'] = parent
     self._triggerProperties = NamespaceElementProperties()
     self._triggerSet = None
@@ -311,8 +320,8 @@ class NamespaceFunction:
     self._name = name
     self._objectFunction = functionLink
 
-  def __call__(self, *args):
-    return self._objectFunction(*args)
+  def __call__(self, *args, **kwargs):
+    return self._objectFunction(*args, **kwargs)
 
 #
 # NamespaceElementProperties
