@@ -122,10 +122,10 @@ class pdftable:
     """
     A simple table based report generator.
     
-    The class provides the following features.
-      Border support for columns, rows.
-      Automatic adjustments of the data when it is unable to fit on one page.
-      Multiple sections of a report handled automatically.
+    The class provides the following features:
+      - Border support for columns, rows.
+      - Automatic adjustments of the data when it is unable to fit on one page.
+      - Multiple sections of a report handled automatically.
     
     @param file: A python file handle used for output
     @param parameterBox: Unused?
@@ -191,7 +191,8 @@ class pdftable:
   # Define a column on the report
   # --------------------------------------------------------------------------
   def addColumn(self, align, minSize, overflow="", highlight=None, 
-                leftBorder=0, rightBorder=0, sectionType="default"):    
+                leftBorder=0, rightBorder=0, topBorder=0, bottomBorder=0,
+                sectionType="default"):    
     """
     Defines a column on the record for the specified section
     
@@ -200,7 +201,9 @@ class pdftable:
                      large for the column width.
     @param highlight: The color of background to use in this column.
     @param leftBorder: The width of the left side border in points.
-    @param rightBorder: The width of the right side border in points.   
+    @param rightBorder: The width of the right side border in points.
+    @param topBorder: The width of the top side border in points.
+    @param bottomBorder: The width of the bottom side border in points.           
     @param sectionType: The name of the section to which this column should be added.
     """
     try:
@@ -213,6 +216,8 @@ class pdftable:
                                        'columnCoords'     :[],
                                        'columnLeftBorder' :[],
                                        'columnRightBorder':[],
+                                       'columnTopBorder'  :[],
+                                       'columnBottomBorder':[],
                                        'headerList':    [[]],                                    
                                     }
       secType = self.definedSectionTypes[sectionType]
@@ -223,6 +228,8 @@ class pdftable:
     secType['columnHighlight'].append(highlight)
     secType['columnLeftBorder'].append(leftBorder)
     secType['columnRightBorder'].append(rightBorder)
+    secType['columnTopBorder'].append(topBorder)
+    secType['columnBottomBorder'].append(bottomBorder)
 
   # ==========================================================================
   # Section level functions
@@ -350,8 +357,11 @@ class pdftable:
       self.drawHorizBorderedBox(leftmargin, boxy, self.width-leftmargin*2,
                                 boxh, subtotalColor, lines=lineWidth * self.scale)
     elif highlighted:
+      # If there is a bottom border we need to not draw over the top
+      # of the previous rows border.  Currently minimum must be 1 point      
+      adjustment = max([self._currentSection['columnBottomBorder'][0],1])
       self.drawHorizBorderedBox(leftmargin, boxy, self.width-leftmargin*2,
-                                boxh, highlightColor, lines=0)
+                                boxh-adjustment, highlightColor, lines=0)
 
     canvas.setFont(font, size)
 
@@ -367,7 +377,7 @@ class pdftable:
       if i < len(self._currentSection['columnCoords'])-1:
         hlx2 = self._currentSection['columnCoords'][i+1][0]-self.columnGap/2.0
       else:
-        hlx2 = self.width-leftmargin*2
+        hlx2 = self.width-leftmargin
       
       # Column highlight support          
       highlightColumn = self._currentSection['columnHighlight'][i]
@@ -380,13 +390,13 @@ class pdftable:
           color = colors.Blacker(highlightColor,.98)
         else:
           color = highlightColumn
-          
-        self.drawHorizBorderedBox(hlx1, boxy - adjust, hlx2-hlx1, boxh, 
+        
+        self.drawHorizBorderedBox(hlx1, boxy - adjust, 
+                                  hlx2-hlx1, boxh, 
                                   color, lines=0)
       
       # Column border support
-      leftBorder = self._currentSection['columnLeftBorder'][i]
-      
+      leftBorder = self._currentSection['columnLeftBorder'][i]      
       if leftBorder:
         canvas.setLineWidth(leftBorder*self.scale)
         canvas.line(hlx1, boxy, hlx1, boxy + boxh)
@@ -395,7 +405,17 @@ class pdftable:
       if rightBorder:
         canvas.setLineWidth(rightBorder*self.scale)
         canvas.line(hlx2, boxy, hlx2, boxy + boxh)
-      
+
+      topBorder =self._currentSection['columnTopBorder'][i]
+      if topBorder:
+        canvas.setLineWidth(topBorder*self.scale)
+        canvas.line(hlx1, boxy + boxh, hlx2, boxy + boxh)
+
+      bottomBorder = self._currentSection['columnBottomBorder'][i]
+      if bottomBorder:
+        canvas.setLineWidth(bottomBorder*self.scale)
+        canvas.line(hlx1, boxy, hlx2, boxy)
+                      
       if col:      
         align= self._currentSection['columnAligns'][i]
         x1, x2 = self._currentSection['columnCoords'][i]
