@@ -1,4 +1,4 @@
-# GNU Enterprise RPC interface - Base for all drivers
+# GNU Enterprise Common Library - RPC interface - Base for all drivers
 #
 # Copyright 2001-2005 Free Software Foundation
 #
@@ -19,21 +19,85 @@
 # write to the Free Software Foundation, Inc., 59 Temple Place
 # - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: Base.py 7009 2005-02-11 17:04:16Z reinhard $
+# $Id: Base.py 7973 2005-09-22 15:13:13Z johannes $
 
-from types import *
-
-import string
 import sys
-import thread
 import traceback
 import urlparse
+import types
 
 from gnue.common.apps import i18n
+from gnue.common.rpc import server
 
 # Indicate that this is not a valid plugin
 __noplugin__ = True
 
+
+# =============================================================================
+# Server Proxy
+# =============================================================================
+
+class ServerProxy:
+  """
+  A ServerProxy provides access to the RPC server, where each attribute is
+  encapsulated by a L{ProxyMethod} instance. Such an instance executes the
+  encapsulated method (the former attribute) on the RPC server.
+  """
+
+  # ---------------------------------------------------------------------------
+  # Constructor
+  # ---------------------------------------------------------------------------
+
+  def __init__ (self, adapter):
+    """
+    Create a new proxy for the given adapter
+
+    @param adapter: L{Server} instance to send all requests to
+    """
+
+    self._adapter = adapter
+
+
+  # ---------------------------------------------------------------------------
+  # attribute access
+  # ---------------------------------------------------------------------------
+
+  def __getattr__ (self, name):
+    """
+    Wrap a L{ProxyMethod} around the requested attribute name.
+
+    @param name: (method)-name to be wrapped within a ProxyMethod
+    """
+
+    if name [0] == '_':
+      raise AttributeError, name
+
+    result = ProxyMethod (self._adapter, name)
+    self.__dict__ [name] = result
+
+    return result
+
+
+  # ---------------------------------------------------------------------------
+  # Nice string represenation
+  # ---------------------------------------------------------------------------
+
+  def __repr__ (self):
+    return "<%s>" % self
+
+  def __str__ (self):
+    return "ServerProxy for %s" % self._adapter._url
+
+
+  # ---------------------------------------------------------------------------
+  # Close the connection to the remote service
+  # ---------------------------------------------------------------------------
+
+  def _destroy (self):
+
+    self._adapter.destroy ()
+
+
 # =============================================================================
 # Client adapter
 # =============================================================================
@@ -42,18 +106,22 @@ class Client:
   """
   Basic client adapter
   """
-  _default_transport = None
-  _default_host      = 'localhost'
-  _default_port      = None
-  _default_path      = '/'
+
+  _default_transport_ = None
+  _default_host_      = 'localhost'
+  _default_port_      = None
+  _default_path_      = '/'
+
+  _serverProxyClass_  = ServerProxy
+
 
   # ---------------------------------------------------------------------------
-  # Initialize object
+  # Constructor
   # ---------------------------------------------------------------------------
 
   def __init__ (self, params):
 
-    checktype (params, DictionaryType)
+    checktype (params, dict)
 
     self._url       = None
     self._transport = None
@@ -66,21 +134,20 @@ class Client:
       # Connection defined as URL
       (self._transport, netloc, self._path, params, query, fragment) \
         = urlparse.urlparse (params ['url'])
-      (self._host, self._port) = split (netloc, ':', 1)
+      (self._host, self._port) = netloc.split (':', 1)
 
     else:
-
       # Connection defined as transport/host/port/path info
-      if params.has_key ('transport'): self._transport = params ['transport']
-      if params.has_key ('host'     ): self._host      = params ['host'     ]
-      if params.has_key ('port'     ): self._port      = params ['port'     ]
-      if params.has_key ('path'     ): self._path      = params ['path'     ]
+      self._transport = params.get ('transport')
+      self._host      = params.get ('host')
+      self._port      = params.get ('port')
+      self._path      = params.get ('path')
 
     # If some info isn't given, fall back to default values
-    if not self._transport: self._transport = self._default_transport
-    if not self._host:      self._host      = self._default_host
-    if not self._port:      self._port      = self._default_port
-    if not self._path:      self._path      = self._default_path
+    if not self._transport: self._transport = self._default_transport_
+    if not self._host:      self._host      = self._default_host_
+    if not self._port:      self._port      = self._default_port_
+    if not self._path:      self._path      = self._default_path_
 
     # Make sure port is an integer
     self._port = int (self._port)
@@ -88,13 +155,19 @@ class Client:
     # Now build the full URL
     self._url = '%s://%s:%d%s' % (self._transport, self._host, self._port,
                                   self._path)
+    self._timeout = params.get ('timeout', 1.0)
 
-    if params.has_key ('timeout'):
-      self._timeout = params ['timeout']
-    else:
-      self._timeout = 1.0
 
-    self.__open = True
+  # ---------------------------------------------------------------------------
+  # Nice string representation
+  # ---------------------------------------------------------------------------
+
+  def __repr__ (self):
+    return "<%s>" % self
+
+  def __str__ (self):
+    return "RPC-CLientAdapter for %s" % self._url
+
 
   # ---------------------------------------------------------------------------
   # Set timeout
@@ -104,190 +177,241 @@ class Client:
 
     self._timeout = timeout
 
-  # ---------------------------------------------------------------------------
-  # Request a (static) proxy object
-  # ---------------------------------------------------------------------------
-
-  def request (self, service):
-
-    return ProxyObject (self, service, False)
 
   # ---------------------------------------------------------------------------
-  # Close the connection
+  # Request a proxy object
   # ---------------------------------------------------------------------------
 
-  def close ():
+  def getServerProxy (self):
     """
-    Close the connection to the server
+    Create a new proxy object to the remote server.
+
+    @returns: L{ServerProxy} instance handling attribute access and method calls
     """
-    if self.__open:
-      self._close ()
+
+    assert gEnter (8)
+    result = self._getServerProxy_ ()
+    assert gLeave (8, result)
+    return result
+
 
   # ---------------------------------------------------------------------------
-  # Clean up
+  # Destroy a client adapter's connection
   # ---------------------------------------------------------------------------
 
-  def __del__ (self):
-
-    if self.__open:
-      self._close ()
-
-  # ---------------------------------------------------------------------------
-  # Run a procedure on the server (abstract)
-  # ---------------------------------------------------------------------------
-
-  def _runMethod (self, method, *args, **params):
-    """
-    execute a remote method
-    """
-    pass
-
-  # ---------------------------------------------------------------------------
-  # Close the server (virtual)
-  # ---------------------------------------------------------------------------
-
-  def _close (self):
+  def destroy (self):
 
     pass
 
-  # ---------------------------------------------------------------------------
-  # Create a dynamic proxy object
-  # ---------------------------------------------------------------------------
 
-  def _createproxy (self, service):
-
-    return ProxyObject (self, service, True)
-
-# =============================================================================
-# Proxy object for clients
-# =============================================================================
-
-class ProxyObject:
+  # ===========================================================================
+  # Virtual functions
+  # ===========================================================================
 
   # ---------------------------------------------------------------------------
-  # Initialize proxy object
+  # Create a new server proxy object
   # ---------------------------------------------------------------------------
 
-  def __init__ (self, adapter, service, dynamic):
+  def _getServerProxy_ (self):
+    """
+    Create a new proxy object to the remote server.
+
+    @returns: server proxy object, handling attribute access and method calls
+    """
+
+    return self._serverProxyClass_ (self)
 
-    self.__adapter = adapter
-    self.__service = service
-    self.__dynamic = dynamic
 
   # ---------------------------------------------------------------------------
-  # Get a (proxy) method
+  # perform a remote procedure call
   # ---------------------------------------------------------------------------
 
-  def __getattr__ (self, attr):
+  def _callMethod_ (self, method, *args, **kwargs):
+    """
+    Call a method with the given arguments on the remote side. Descendants
+    override this function to do the actual remote procedure call.
 
-    if attr[0] == '_':
-      raise AttributeError, attr
+    @param method: name of the method to call
+    @param args: tuple with all positional arguments
+    @param kwargs: dictionary with all keyword arguments
 
-    method = ProxyMethod (self.__adapter, self.__service + '.' + attr)
-    self.__dict__ [attr] = method
-    return method
+    @return: result of the remote procedure call
+    """
 
-  # ---------------------------------------------------------------------------
-  # Set an object attribute (must start with '_')
-  # ---------------------------------------------------------------------------
+    raise NotImplementedError
 
-  def __setattr__ (self, attr, value):
-
-    # FIXME: what do we need this for?
-
-    if attr[0] == '_':
-      self.__dict__[attr] = value
-    else:
-      raise AttributeError, attr
-
-  # ---------------------------------------------------------------------------
-  # Clean up
-  # ---------------------------------------------------------------------------
-
-  def __del__ (self):
-
-    if self.__dynamic:
-      self.__adapter._runMethod (self.__service + '._close')
 
 # =============================================================================
 # Proxy method for clients
 # =============================================================================
 
 class ProxyMethod:
+  """
+  A ProxyMethod provides a callable for a method in a RPC server. This method
+  will be executed by the given L{Client} adapter.
+  """
 
   # ---------------------------------------------------------------------------
   # Initialize proxy method
   # ---------------------------------------------------------------------------
 
   def __init__ (self, adapter, methodname):
+    """
+    @param adapter: L{Client} instance implementing a L{_callMethod_} function
+    @param methodname: name of the method be called
+    """
 
     self._adapter    = adapter
     self._methodname = methodname
 
+
   # ---------------------------------------------------------------------------
   # Run the method
   # ---------------------------------------------------------------------------
 
   def __call__ (self, *args, **params):
+    """
+    Execute the wrapped RPC-Method via L{Client} adapter using the given
+    positional and keyword-arguments.
+    
+    Note: not all RPC adapter can handle keyword arguments, so take care.
+    """
+
+    assert gEnter (8)
+    result = self._adapter._callMethod_ (self._methodname, *args, **params)
+    assert gLeave (8, result)
+    return result
+
+
+  # ---------------------------------------------------------------------------
+  # Attribute access
+  # ---------------------------------------------------------------------------
+
+  def __getattr__ (self, name):
+    """
+    Support for nested methods, e.g. system.listMethods.
+    @param name: name of the attribute in question
+
+    @returns: L{ProxyMethod} instance pointing to the nested method
+    """
+
+    return ProxyMethod (self._adapter, "%s.%s" % (self._methodname, name))
+
+
+  # ---------------------------------------------------------------------------
+  # Nice string representation
+  # ---------------------------------------------------------------------------
+
+  def __repr__ (self):
+    return "<%s>" % self
+
+  def __str__ (self):
+    return "ProxyMethod '%s' of %s" % (self._methodname, self._adapter)
+
 
-    gDebug (3, "%s (%s)" \
-            % (self._methodname,
-               string.join ([repr (x) for x in args] + \
-                            ["%s = %s" % (x [0], repr (x [1])) \
-                             for x in params.items ()], ', ')))
-    return self._adapter._runMethod (self._methodname, *args, **params)
 
 # =============================================================================
-# Server adapter
+# Base class for server adapters
 # =============================================================================
 
 class Server:
   """
-  Basic server adapter
+  Basic server adapter.
+
+  @cvar _default_transport_: default transport to use if no transport parameter
+    is given.
+  @cvar _default_port_: default port to use if no port parameter is given
+
+  @ivar instance: the python instance to be served by this server
+  @ivar allowed_hosts: a sequence of hosts (name/ip-addresses) which are
+    allowed to connect to this server. An empty sequence means 'no restriction'
   """
-  _default_transport = None
-  _default_port      = None
+
+  _default_transport_ = None
+  _default_port_      = None
+
 
   # ---------------------------------------------------------------------------
   # Initialize server object
   # ---------------------------------------------------------------------------
 
-  def __init__ (self, rpcdef, bindings, params):
-    self._bindings = bindings
-    self._rpcdef = rpcdef
+  def __init__ (self, service, parameters):
+    """
+    Create a new server adapter serving a given python object.
 
-    checktype (params, DictionaryType)
+    @param service: python object to be served
+    @param parameters: dictionary of parameters for the server adapter
+    """
 
-    self._transport = None
-    self._port      = None
+    checktype (service, types.InstanceType)
+    checktype (parameters, dict)
 
-    if params.has_key ('transport'): self._transport = params ['transport']
-    if params.has_key ('port'     ): self._port      = params ['port'     ]
+    self.instance   = service
 
-    # If some info isn't given, fall back to default values
-    if not self._transport: self._transport = self._default_transport
-    if not self._port:      self._port      = self._default_port
+    self._transport = parameters.get ('transport', self._default_transport_)
+    self._port      = parameters.get ('port', self._default_port_)
+
+    if not self._port:
+      raise server.AdapterConfigurationError, \
+          u_("Required parameter 'port' not supplied")
+
+    self._port   = int (self._port)
+    self._bindto = parameters.get ('bindto', '')
+
+    self.allowed_hosts = parameters.get ('allowed_hosts', '').split (',')
+
+
+  # ---------------------------------------------------------------------------
+  # Nice string representation
+  # ---------------------------------------------------------------------------
+
+  def __repr__ (self):
+
+    return "<RPC-Server serving '%r' at %x>" % (self.instance, id (self))
 
-    # Make sure port is an integer
-    self._port = int (self._port)
 
   # ---------------------------------------------------------------------------
   # Start server
   # ---------------------------------------------------------------------------
 
   def serve (self):
+    """
+    Start the server.
+    """
+
+    assert gEnter (8)
+    result = self._serve_ ()
+    assert gLeave (8, result)
+    return result
+
+
+  # ---------------------------------------------------------------------------
+  # Shut down the server
+  # ---------------------------------------------------------------------------
+
+  def shutdown (self):
+    """
+    """
+
+    assert gEnter (8)
+    self._shutdown_ ()
+    assert gLeave (8)
+
+
+  # ---------------------------------------------------------------------------
+  # Virtual methods to be implemented by descendants
+  # ---------------------------------------------------------------------------
+
+  def _serve_ (self):
+    """
+    Start serving the instance.
+    """
+
     pass
 
-  # ---------------------------------------------------------------------------
-  # Start server as new thread
-  # ---------------------------------------------------------------------------
-
-  def serveAsNewThread (self):
-    thread.start_new_thread (self.serve, ())
 
   # ---------------------------------------------------------------------------
-  # Return an exception
-  # ---------------------------------------------------------------------------
 
-  def raiseException (self, exception, message, event=None):
-    raise exception, message
+  def _shutdown_ (self):
+
+    pass
