@@ -36,19 +36,19 @@ from gnue.common.setup import ChangeLog
 # -----------------------------------------------------------------------------
 
 try:
-  if sys.hexversion < 0x02000000:
+  if sys.hexversion < 0x02030000:
     raise AttributeError
 except AttributeError:
   print "-" * 70
   print """
   You are running Python %s.
 
-  GNU Enterprise requires at least Python 2.0 (recommended: 2.1+).
+  GNU Enterprise requires at least Python 2.3.
   If you have a later version installed, you should run setup.py
-  against that version. For example, if you have Python 2.2
+  against that version. For example, if you have Python 2.3
   installed, you may need to run:
 
-       python2.2 setup.py
+       python2.3 setup.py
 """ % string.split(sys.version)[0]
   print "-" * 70
   sys.exit (1)
@@ -122,26 +122,16 @@ class install (distutils.command.install.install):
 
     # install translations
     if os.path.isdir ('po'):
-
-      # find out domain
-      domain = None
+      # copy files
       for f in os.listdir ('po'):
-        if f [-4:] == '.pot':
-          domain = f [:-4]
-
-      if not domain:
-        print "warning: cannot determine domain, not installing translations"
-      else:
-        # copy files
-        for f in os.listdir ('po'):
-          if f [-4:] == '.gmo':
-            src = os.path.join ('po', f)
-            dst = os.path.join (self.install_data, 'share', 'locale', f [:-4],
-                                'LC_MESSAGES')
-            self.mkpath (dst)
-            dst = os.path.join (dst, domain + '.mo')
-            self.copy_file (src, dst)
-            self.__outputs.append (dst)
+        if f [-4:] == '.gmo':
+          src = os.path.join ('po', f)
+          dst = os.path.join (self.install_data, 'share', 'locale', f [:-4],
+                              'LC_MESSAGES')
+          self.mkpath (dst)
+          dst = os.path.join (dst, _setup.package + '.mo')
+          self.copy_file (src, dst)
+          self.__outputs.append (dst)
 
     distutils.command.install.install.run (self)
 
@@ -290,6 +280,9 @@ class GSetup:
                    }
 
     _setup.set_params (setup_params)
+
+    # make package available
+    self.package = setup_params ["name"]
     
     # handle version suffix
     try:
